@@ -3,4 +3,6 @@
 from .modes import Mode
 from .molecules import Molecule
 from .aggregates import Aggregate
-from .pdb import PDBFile
\ No newline at end of file
+from .pdb import PDBFile
+from .aggregate_states import ElectronicState
+from .aggregate_states import VibronicState
\ No newline at end of file
