@@ -48,10 +48,11 @@ class DistributedConfiguration:
                     
         self.parallel_level = 0
         
-        if self.size > 1:
-            self.parallel_level = 1
+        #if self.size > 1:
+        #    self.parallel_level = 1
             
         self.inparallel = False
+        self.parallel_region = 0
         
         self.silent = True
         
@@ -97,6 +98,9 @@ class DistributedConfiguration:
             else:
                 import atexit
                 atexit.register(call_finish)
+                
+        # this counting is independent of whether we have MPI
+        self.parallel_region += 1
         
 
     def finish_parallel_region(self):
@@ -114,6 +118,7 @@ class DistributedConfiguration:
         if self.parallel_level < 0:
             raise Exception()
             
+        self.parallel_region -= 1
         
     def print_info(self):
         if self.rank == 0:
@@ -140,7 +145,11 @@ class DistributedConfiguration:
         on the process with rank = 0
 
         """ 
-        # only in parallel_level == 1 we share the work
+
+        if self.parallel_region < 1:
+            raise Exception("This code has to be run from a declared parallel_region")
+ 
+         # only in parallel_level == 1 we share the work
         if self.parallel_level != 1:
             return A
 
@@ -163,7 +172,10 @@ class DistributedConfiguration:
         processes.
 
         """ 
-
+        
+        if self.parallel_region < 1:
+            raise Exception("This code has to be run from a declared parallel_region")
+        
         # only in parallel_level == 1 we share the work
         if self.parallel_level != 1:
             return 
@@ -181,7 +193,9 @@ class DistributedConfiguration:
     def bcast(self, value, root=0):
         #if self.parallel_level != 1:
         #    return value
-            
+        if self.parallel_region < 1:
+            raise Exception("This code has to be run from a declared parallel_region")
+        
         return self.comm.bcast(value, root=root)
 
             
@@ -194,7 +208,7 @@ def start_parallel_region():
     """
     from .managers import Manager
     dc = Manager().get_DistributedConfiguration()
-    #dc.start_parallel_region()
+    dc.start_parallel_region()
     if dc.rank != 0:
         Manager().log_conf.verbosity -= 2
         Manager().log_conf.fverbosity -= 2
@@ -209,7 +223,7 @@ def close_parallel_region():
     """     
     from .managers import Manager
     dc = Manager().get_DistributedConfiguration()
-    #dc.finish_parallel_region()
+    dc.finish_parallel_region()
     if dc.rank != 0:
         Manager().log_conf.verbosity += 2
         Manager().log_conf.fverbosity += 2
@@ -239,9 +253,13 @@ def block_distributed_range(start, stop):
             end of the range
 
     """
+    
     from .managers import Manager
     # we share the work only in parallel_level == 1  
     config = Manager().get_DistributedConfiguration()
+
+    if config.parallel_region < 1:
+        raise Exception("This code has to be run from a declared parallel_region")
         
     if config.parallel_level==1:
         
@@ -278,7 +296,10 @@ def block_distributed_list(dlist, return_index=False):
     from .managers import Manager
     # we share the work only in parallel_level == 1  
     config = Manager().get_DistributedConfiguration()
-        
+ 
+    if config.parallel_region < 1:
+        raise Exception("This code has to be run from a declared parallel_region")
+ 
     if config.parallel_level==1:
 
         config.inparallel_entered = True
@@ -317,7 +338,10 @@ def block_distributed_array(array, return_index=False):
     from .managers import Manager
     # we share the work only in parallel_level == 1  
     config = Manager().get_DistributedConfiguration()
-        
+
+    if config.parallel_region < 1:
+        raise Exception("This code has to be run from a declared parallel_region")
+    
     if config.parallel_level==1:
 
         config.inparallel_entered = True
@@ -350,7 +374,7 @@ def block_distributed_array(array, return_index=False):
 
 def collect_block_distributed_data(containers, setter_function,
                                   retriever_function, tags=None):
-    """Collects distributed data into a container container on rank 0 nod
+    """Collects distributed data into a container on rank 0 nod
     
     Collects the "data" properties of the objects of the 
     data_class type into a container on the rank = 0 nod.
@@ -380,6 +404,9 @@ def collect_block_distributed_data(containers, setter_function,
     from .managers import Manager
     # we share the work only in parallel_level == 1  
     config = Manager().get_DistributedConfiguration()
+
+    if config.parallel_region < 1:
+        raise Exception("This code has to be run from a declared parallel_region")
     
     if config.parallel_level==1:
      
@@ -531,19 +558,162 @@ def _calculate_ranges_array(config, array):
     return _calculate_ranges(config, start, stop)
       
 
+import threading
+
+_sentinel = object()
+
+def _send_to_other_by_MPI(config, i, dest, finish=False):
+    
+    to_send = numpy.zeros(2, dtype=int)
+    to_send[0] = i
+    if finish:
+        to_send[1] = 1
+    else:
+        to_send[1] = 0
+        
+    # send it
+    config.comm.Send(to_send, dest=dest)
+
+
+def _receive_from_MPI(config, source):
+    
+    # receive it
+    data = numpy.zeros(2, dtype=int)
+    config.comm.Recv(data, source=source)
+    
+    if data[1] == 1:
+        return None
+    else:
+        return data[0]
+
+
+
+class RangeDistributor(threading.Thread):
+    
+    def __init__(self, start, stop, queue, config):
+        threading.Thread.__init__(self)
+        self.threadID = 0
+        self.name = "Range Distributor"
+        self.counter = 0
+        self.strt = start
+        self.stop = stop
+        
+        self.queue = queue
+        self.config = config
+    
+    def run(self):
+        
+        nproc = self.config.size
+        
+        l = 0
+        for val in range(self.strt, self.stop):
+            
+            #print("In loop:", val)
+            
+            if l == 0:
+                # send to rank 0 via a queue
+                #print("putting", val, "into queue")
+                self.queue.put(val)
+            else:
+                # send to others via MPI
+                #print("sending", val, "by MPI")
+                _send_to_other_by_MPI(self.config, val, l)
+            
+            l += 1
+            if l >= nproc:
+                l = 0
+                
+        for k in range(nproc):
+            
+            if l == 0:
+                #print("Putting finish signal to queue")
+                # send finish signal to rank 0 via a queue
+                self.queue.put(_sentinel)
+            else:
+                #print("Sending finish signal by MPI")
+                # send finish signal to others via MPI
+                _send_to_other_by_MPI(self.config, 0, l, finish=True)
+            
+            l += 1
+            if l >= nproc:
+                l = 0           
+        
+        
+def _get_next_from_thread(queue):
+
+    val = queue.get()
+        
+    if val is _sentinel:
+        
+        return None
+
+    return val
+
+    
+def _get_next_from_MPI(config, source):
+    
+    # receive value through MPI
+    return _receive_from_MPI(config, source)
+
+
+
 def asynchronous_range(start, stop):
     """Range distributing numbers asynchronously among processes
     
     """
     from .managers import Manager
     config = Manager().get_DistributedConfiguration()
-    if config.parallel_level==1:   
-        pass
+    
+    if config.parallel_region < 1:
+        raise Exception("This code has to be run from a declared parallel_region")    
+    
+    if config.parallel_level==1:  
+        
+        from queue import Queue
+        q = Queue()
+        
+        if config.rank == 0:
+            
+            # start a separate thread which will devide job between processes
+            # including the present one
+            rd = RangeDistributor(start, stop, q, config)
+            rd.start()
+            
+        not_done = True
+        while not_done:
+            
+            if config.rank == 0:
+                
+                next_val = _get_next_from_thread(q)
+            
+            else:
+                
+                next_val = _get_next_from_MPI(config, 0)
+                
+            #print("Got: ", next_val, "in", config.rank)
+            if next_val is not None:
+                #print("Yielding", next_val, "in", config.rank)
+                yield next_val
+        
+            else:
+                not_done = False
+        
+        if config.rank == 0:
+            
+            # at the end of the work, the Thread rejoins the main process
+            rd.join()
+            
+            
+    
     else:
 
-        return range(start, stop)
+        # serial implementation of the same
+        for aa in range(start, stop):
+            yield aa
     
 
+
+
 def _parallel_function_wrapper(func, root):
     # FIXME: return a wrapped function with parameter broadcasting
     
@@ -580,7 +750,7 @@ class parallel_function:
     Examples
     --------
 
-#    These examples, when tested, will be run serial mode and the test will
+#    These examples, when tested, will be run in serial mode and the test will
 #    only confirm that the serial version works
 #    
 #    
@@ -640,7 +810,8 @@ class parallel_function:
         
         manager = Manager()
         self.dc = manager.get_DistributedConfiguration()
-
+        if self.dc.parallel_region < 1:
+            raise Exception("This code has to be run from a declared parallel_region")
     
     def __enter__(self):
         """All except of the leader are put on hold
@@ -690,4 +861,4 @@ class parallel_function:
             wait_for_work = self.dc.bcast(wait_for_work,
                                           root=self.leader)
             
-        
\ No newline at end of file
+        
