@@ -16,6 +16,7 @@ from ...core.time import TimeAxis
 from .correlationfunctions import CorrelationFunction
 from .correlationfunctions import FTCorrelationFunction
 from ...core.units import kB_int
+from ...core.units import convert
 
 #from .correlationfunctions import c2h
 
@@ -114,7 +115,8 @@ class SpectralDensity(DFunction, UnitsManaged):
 
     """
 
-    energy_params = ("reorg", "omega", "freq")
+    energy_params = ("reorg", "omega", "freq", "fcp", "g_FWHM", "l_FWHM",\
+                     "freq1", "freq2", "gamma")
     analytical_types = ("OverdampedBrownian")
 
     def __init__(self, axis=None, params=None, values=None):
@@ -177,7 +179,8 @@ class SpectralDensity(DFunction, UnitsManaged):
                     prms = {}
                     for key in params.keys():
                         if key in self.energy_params:
-                            prms[key] = self.convert_energy_2_internal_u(params[key])
+                            prms[key] = \
+                            self.convert_energy_2_internal_u(params[key])
                         else:
                             prms[key] = params[key]
         
@@ -195,6 +198,18 @@ class SpectralDensity(DFunction, UnitsManaged):
         
                     self._make_underdamped_brownian(prms, values)
                     
+                elif ftype == "Underdamped":
+           
+                    self._make_underdamped(params)
+                    
+                elif ftype == "B777":
+                    
+                    self._make_B777(prms)
+                    
+                elif ftype == "CP29":
+                    
+                    self._make_CP29_spectral_density(params, values)
+                    
                 elif ftype == "Value-defined":
         
                     self._make_value_defined(prms, values=values)
@@ -209,7 +224,11 @@ class SpectralDensity(DFunction, UnitsManaged):
         """ Sets the Overdamped Brownian oscillator spectral density
 
         """
-        ctime = params["cortime"]
+        try:
+            ctime = params["cortime"]
+        except:
+            gamma = params["gamma"]
+            ctime = 1/gamma
         lamb = params["reorg"]
 
         # protect calculation from units management
@@ -242,8 +261,12 @@ class SpectralDensity(DFunction, UnitsManaged):
         # protect calculation from units management
         with energy_units("int"):
             omega = self.axis.data
-            cfce = (lamb*ctime)*omega/((omega-omega0)**2 + (ctime)**2) \
-                  +(lamb*ctime)*omega/((omega+omega0)**2 + (ctime)**2)
+            #cfce = (lamb*ctime)*omega/((omega-omega0)**2 + (ctime)**2) \
+            #      +(lamb*ctime)*omega/((omega+omega0)**2 + (ctime)**2)
+            cfce = (2.0*lamb*ctime)*(omega0**2)*\
+                  (omega/(((omega**2)-(omega0**2))**2 + (omega**2)*(ctime**2))) 
+                  #+omega/((omega**2+omega0**2)**2 + (omega**2)*(ctime)**2))
+
 
         if values is not None:
             self._add_me(self.axis, values)
@@ -259,6 +282,150 @@ class SpectralDensity(DFunction, UnitsManaged):
         for i in range(2):
             self.lim_omega[i] += lim_omega[i]
             
+    # See Valkunas, Abramavicius, Mančal, 2013, Wiley-VCH  
+    def _make_underdamped(self, params, values=None):
+        SPEED_OF_LIGHT = 2.99*(10**8)
+ 
+        # use the units in which params was defined
+        omega0 = params["freq"]
+        lamb = params["reorg"]
+        gamma = params["gamma"]
+        
+        # protect calculation from units management
+        with energy_units("int"):
+            omega = self.axis.data
+            cfce = 2*(lamb*omega*gamma*omega0**2)/((omega**2 - \
+                     omega0**2)**2 + (gamma*omega)**2)
+
+        if values is not None:
+            self._make_me(self.axis, values)
+        else:
+            self._make_me(self.axis, cfce)
+
+        # this is in internal units
+        self.lamb = lamb            
+        self.lim_omega = numpy.zeros(2)
+        self.lim_omega[0] = 0.0
+        self.lim_omega[1] = 4*(gamma*(omega0**2))/((omega0**2)**2)
+        
+    # See Renger, Journal of Chemical Physics 2002
+    # See Jang, Newton, Silbey, J Chem Phys. 2007 for alternate form
+    # (See Kell et al, 2013, J. Phys. Chem. B.) 
+    def _make_B777(self, params, values=None):
+        
+        with energy_units("int"):
+            omega = self.axis.data
+            cfce=0
+
+            if not params["alternative_form"]:
+
+                try:
+                    ss = [params['s1'], params['s2']]
+                    freq = [params["freq1"], params["freq2"]]
+                except:
+                    ss = [0.8, 0.5]
+                    freq = [convert(0.56, "1/cm", "int"), convert(1.9, "1/cm", "int")]
+
+                for ii in range(2):
+                    cfce = cfce+\
+                    (ss[ii]/(numpy.math.factorial(7)*2*(freq[ii]**4)))*\
+                    (omega**3)*(numpy.exp(-numpy.abs(omega/freq[ii])**0.5))
+                # Converts the form of the spectral density to the one used in Quantarhei
+                cfce = cfce * (omega**2)
+                # Brings the reorganisation energy to the lit value of 102
+                cfce = cfce * 3.204215
+                print("Renger form of spec dens used")
+
+            else:
+
+                #This form is taken from Jang, Newton, Silbey, J Chem Phys. 2007.
+                #It gives a polynomial form of the B777 spectral density
+                try: 
+                    omega1c = convert(params['om1'], "1/cm", "int")
+                    omega2c = convert(params['om2'], "1/cm", "int")
+                    omega3c = convert(params['om3'], "1/cm", "int")
+                except:
+                    omega1c = convert(170, "1/cm", "int")
+                    omega2c = convert(34, "1/cm", "int")
+                    omega3c = convert(69, "1/cm", "int")
+
+                with energy_units("int"):
+                    # (omega/(numpy.abs(omega))) in the second term ensures
+                    # proper treatment of -ve frequencies
+                    omega = self.axis.data
+                    cfce = \
+                    0.22*omega*numpy.exp(-numpy.abs(omega/omega1c))+\
+                    0.78*(omega/(numpy.abs(omega)))*((omega**2)/omega2c)*numpy.exp(-numpy.abs(omega/omega2c))+\
+                    0.31*((omega**3)/(omega3c**2))*numpy.exp(-numpy.abs(omega/omega3c))
+                cfce = cfce * 3.058187
+                print('Alternate form of spec dens used')
+
+            # This brings the reorganisation energy up to the literature value of 102
+            #cfce = cfce * 3.19
+
+        if values is not None:
+            self._make_me(self.axis, values)
+        else:
+            self._make_me(self.axis, cfce)
+
+        self.lamb = params["reorg"]            
+        self.lim_omega = numpy.zeros(2)
+        self.lim_omega[0] = 0.0
+        self.lim_omega[1] = 0.0
+        
+    def _make_CP29_spectral_density(self, params, values = None):
+    #This pectral density is based on the one calculated from FLN by 
+    #Rätsep et al. J. Phys. Chem. B 2008, 112, 110-118. It consist of a 
+    #Gaussian on the low-frequency side and a Lagrangian on the high-frequency 
+    #side, with the change point between the functions at 22 per cm. The 
+    #spectral density is scaled by the user-supplied reorg energy and 
+    #prefactors are therefore ignored in the analytical calculations.
+    #default values (1/cm) are: function_change_point=22, g_FWHM = 20,
+    #l_FWHM=60 
+        
+        try:
+            function_change_point = params['fcp']
+            g_FWHM = params['g_FWHM']
+            l_FWHM = params['l_FWHM']
+        except:
+            function_change_point = self.manager.iu_energy(22,
+                                       units="1/cm")
+            g_FWHM = self.manager.iu_energy(20,
+                                       units="1/cm")
+            l_FWHM = self.manager.iu_energy(60,
+                                       units="1/cm")
+            
+        lamb = params["reorg"]
+        cfce = numpy.zeros(self.axis.data.shape)
+       
+        with energy_units("int"):
+            omega = self.axis.data
+            g = numpy.where(numpy.abs(omega) < function_change_point)
+            l = numpy.where(numpy.abs(omega) >= function_change_point)
+            cfce[g] = numpy.exp((-(numpy.abs(omega[g]) - \
+                function_change_point)**2)/(2*0.1803*g_FWHM**2))      
+            cfce[l] = 1/((numpy.abs(omega[l]) - \
+                function_change_point)**2 + (l_FWHM/2)**2)      
+            cfce[g] = cfce[g] * (numpy.amax(cfce[l])/numpy.amax(cfce[g]))     
+            cfce[numpy.where(omega < 0)] = -1*cfce[numpy.where(omega < 0)]     
+            cfce[numpy.isclose(omega, 0, atol=1e-05)] = 0
+            
+        if values is not None:
+            self._make_me(self.axis, values)
+            print('spectral density made from correlation function values')
+
+        else:
+            self._make_me(self.axis, cfce)
+            with energy_units("int"):
+                meareorg = self.measure_reorganization_energy()
+            cfce = (lamb/meareorg)*cfce
+            self._make_me(self.axis, cfce)
+
+        self.lamb = lamb     
+        self.lim_omega = numpy.zeros(2)
+        self.lim_omega[0] = 0.0
+        self.lim_omega[1] = 0.0
+            
     def _make_value_defined(self, values=None):
         """ Value defined spectral density
 
@@ -300,7 +467,7 @@ class SpectralDensity(DFunction, UnitsManaged):
         """Inplace addition of two correlation functions
         
         """  
-        self.add_to_data(other)       
+        self.add_to_data2(other)       
         return self
     
             
@@ -329,10 +496,44 @@ class SpectralDensity(DFunction, UnitsManaged):
            
 
         else:
-            raise Exception("In addition, functions have to share"
-                            +" the same FrequencyAxis object")
+            raise Exception("In the operation of addition, functions "
+                           +"have to share the same FrequencyAxis object")
 
 
+    def add_to_data2(self, other):
+        """Addition of data from a specified SpectralDensity to this object
+        
+        """
+        if self == other:
+            ocor = SpectralDensity(other.axis, other.params)
+        else:
+            ocor = other
+            
+        t1 = self.axis
+        t2 = ocor.axis
+        if t1 == t2:
+            
+            self.data += ocor.data
+            self.lamb += ocor.lamb  # reorganization energy is additive
+            #if ocor.cutoff_time > self.cutoff_time: 
+            #    self.cutoff_time = ocor.cutoff_time  
+                
+            #if self.temperature != ocor.temperature:
+            #    raise Exception("Cannot add two correlation functions "
+            #                   +"on different temperatures")
+    
+
+            for p in ocor.params:
+                self.params.append(p)
+            
+            self._is_composed = True
+            self._is_empty = False
+            
+
+        else:
+            raise Exception("In the operation of addition, functions "
+                           +"have to share the same FrequencyAxis object")
+            
     def is_analytical(self):
         """Returns `True` if analytical
 
@@ -381,8 +582,10 @@ class SpectralDensity(DFunction, UnitsManaged):
         return SpectralDensity(self.axis, self.params)
 
 
-    def get_CorrelationFunction(self, temperature=None):
-        """Returns correlation function corresponding to the spectral density
+    def get_CorrelationFunction(self, temperature=None, ta=None):
+        """Returns correlation function corresponding to the spectral density.
+        If a TimeAxis object is included, the CorrelationFunction
+        object will be returned with that TimeAxis instance as its time axis.
 
         """
 
@@ -395,6 +598,13 @@ class SpectralDensity(DFunction, UnitsManaged):
             params.append(newdict)
 
         time = self.axis.get_TimeAxis()
+        
+        if ta is not None:
+            if numpy.all(numpy.isclose(ta.data, time.data, 1e-5)):
+                time = ta
+            else:
+                raise Exception('The provided TimeAxis does not have the same\
+                                data as the Fourier transformed axis')
 
         # everything has to be protected from change of units
         with energy_units("int"):
