@@ -3,7 +3,8 @@
 
 import unittest
 import numpy
-import h5py
+#import h5py
+import tempfile
 
 
 """
@@ -38,15 +39,17 @@ class TestValueAxis(unittest.TestCase):
         
         """
         
-        with h5py.File("test_file_ValueAxes",driver="core", 
-                           backing_store=False) as f:
-            
+        #with h5py.File("test_file_ValueAxes",driver="core", 
+        #                   backing_store=False) as f:
+        with tempfile.TemporaryFile() as f:
+        
             ta = ValueAxis(0.0, 1000, 0.1)
         
-            ta.save(f, test=True)
+            ta.save(f) #, test=True)
+            f.seek(0)
         
             tb = ValueAxis()
-            tb.load(f, test=True)
+            tb = tb.load(f) #, test=True)
         
         numpy.testing.assert_array_equal(ta.data,tb.data)
         self.assertEqual(ta.length,tb.length)
